@@ -60,7 +60,6 @@ class Receiver(object):
         and return the first one found as "supplied remote".
         '''
 
-        # pylint: disable-msg=W0141
         remotes = self.__compatible_remotes or []
         remotes = map(remotes_db.get, remotes)
         remotes = filter(None, remotes)
@@ -357,6 +356,7 @@ class RemotesParser(object):
 
         self.__current_line = 0
         self.__filename = None
+        self.__database = None
 
         self.__next_remote()
 
@@ -509,6 +509,15 @@ class RemotesParser(object):
     current_block = property(lambda self: len(self.__blocks) and self.__blocks[-1])
     toplevel_block = property(lambda self: len(self.__blocks) and self.__blocks[0])
 
+class KeyCodeCategory(object):
+    '''
+    Container for various key-code categories.
+    '''
+
+    DEFAULT = _('Default Namespace')
+    CUSTOM = _('Custom Key Code')
+    ELISA = _('Elisa Compatible')
+
 def create_commands_table():
     '''
     Create a dict mapping the command name to full details,
@@ -520,7 +529,7 @@ def create_commands_table():
 
     #TODO: Add other keys (and alternatives key names):
     commands_by_category = [
-        (_('Default Namespace'), (
+        (KeyCodeCategory.DEFAULT, (
             ('BTN_LEFT',           'move_up_key',        _('Move Up')),
             ('BTN_MODE',           'move_up_key',        _('Move Up')),
             ('BTN_RIGHT',          'move_up_key',        _('Move Up')),
@@ -641,7 +650,7 @@ def create_commands_table():
         )),
 
         # Key names used by Elisa (as of 2007-02-12):
-        (_('Elisa Compatible'), (
+        (KeyCodeCategory.ELISA, (
             ('GO_UP',              'move_up_key',              _('Move Up')),
             ('GO_DOWN',            'move_down_key',            _('Move Down')),
             ('GO_LEFT',            'move_left_key',            _('Move Left')),
@@ -671,7 +680,7 @@ def create_commands_table():
         # standard names, and we should probably warn about (or ignore as
         # broken) non-standard key names, because applications are unlikely
         # to work with those broken lircd.conf files.
-        (_('Custom Key Code'), (
+        (KeyCodeCategory.CUSTOM, (
             ('CH_UP',              'ch_up_key',              _('Channel Up')),
             ('CH_DOWN',            'ch_down_key',            _('Channel Down')),
             ('<<',                 'rewind_key',             _('Rewind')),
