@@ -19,7 +19,7 @@
 The main window of the application.
 '''
 
-import dbus, gobject, gtk, gtk.gdk, gtk.glade, locale, logging, os
+import dbus, gobject, gtk, gtk.gdk, gtk.glade, httplib, locale, logging, os
 
 from gettext               import gettext as _
 from gnome_lirc_properties import backend, config, lirc, model, hardware, policykit, net
@@ -30,9 +30,7 @@ from gnome_lirc_properties.ui.ProgressWindow        import ProgressWindow
 from gnome_lirc_properties.ui.ReceiverChooserDialog import ReceiverChooserDialog
 
 class RemoteControlProperties(object):
-    '''
-    The main window.
-    '''
+    '''The main window.'''
 
     def __init__(self, glade_xml):
         # Prevent UI changes from being written back to configuration files:
@@ -60,9 +58,7 @@ class RemoteControlProperties(object):
         self.__configuration_level = 0
 
     def __setup_models(self):
-        '''
-        Initialize model objects of the dialog.
-        '''
+        '''Initialize model objects of the dialog.'''
 
         # pylint: disable-msg=W0201,E1101
 
@@ -81,9 +77,7 @@ class RemoteControlProperties(object):
         self.__update_remotes_db()
 
     def __update_remotes_db(self):
-        '''
-        Fills the database of remote controls with information.
-        '''
+        '''Fills the database of remote controls with information.'''
 
         was_using_supplied = self.use_supplied_remote
         selected_remote = self.selected_remote
@@ -97,14 +91,14 @@ class RemoteControlProperties(object):
         self.__remote_vendors.load(self.__remotes_db)
 
         if 0 == self.__configuration_level:
+            # pylint: disable-msg=E1103
+
             self.selected_remote = (
                 (was_using_supplied and self.supplied_remote) or
                 (selected_remote and self.__remotes_db.get(selected_remote.name)))
 
     def __setup_key_listener(self):
-        '''
-        Initialize the key-listener and related widgets.
-        '''
+        '''Initialize the key-listener and related widgets.'''
 
         # pylint: disable-msg=W0201,E1101
 
@@ -113,6 +107,8 @@ class RemoteControlProperties(object):
         self.__key_listener.connect('key-pressed', self.__on_lirc_key_pressed)
 
     def __lookup_widgets(self):
+        '''Initialize widget attributes from Glade file.'''
+
         # pylint: disable-msg=W0201
 
         widget_list = (
@@ -173,9 +169,7 @@ class RemoteControlProperties(object):
         size_group.add_widget(self.__label_preview_result)
 
     def __setup_product_lists(self):
-        '''
-        Initialize widgets with product listings.
-        '''
+        '''Initialize widgets with product listings.'''
 
         self.__combo_receiver_vendor_list.set_model(self.__receiver_vendors)
         self.__combo_receiver_vendor_list.set_active(0)
@@ -187,9 +181,7 @@ class RemoteControlProperties(object):
         self._on_radiobutton_supplied_remote_toggled()
 
     def __setup_authorization(self):
-        '''
-        Initialize authorization facilities.
-        '''
+        '''Initialize authorization facilities.'''
 
         # pylint: disable-msg=W0201
 
@@ -202,9 +194,7 @@ class RemoteControlProperties(object):
         self._set_widgets_locked(not granted)
 
     def __restore_hardware_settings(self):
-        '''
-        Restore hardware settings from configuration files.
-        '''
+        '''Restore hardware settings from configuration files.'''
 
         assert 0 != self.__configuration_level
 
@@ -227,9 +217,7 @@ class RemoteControlProperties(object):
         self.use_supplied_remote = (self.supplied_remote == self.selected_remote)
 
     def __on_lirc_changed(self, listener):
-        '''
-        Handle state changes of the LIRC key listener.
-        '''
+        '''Handle state changes of the LIRC key listener.'''
 
         if listener.connected:
             self.__label_preview_result.show()
@@ -246,13 +234,15 @@ class RemoteControlProperties(object):
 
     # pylint: disable-msg=W0613,R0913
     def __on_lirc_key_pressed(self, listener, remote, repeat, name, code):
-        '''
-        Handle key presses reported by the LIRC key listener.
-        '''
+        '''Handle key presses reported by the LIRC key listener.'''
 
         display_name = lirc.KeyCodes.get_display_name(name)
-        display_name = gobject.markup_escape_text(display_name)
-        self.__label_preview_result.set_markup('<b>%s</b>' % display_name)
+        category = lirc.KeyCodes.get_category(name)
+
+        args = map(gobject.markup_escape_text, (display_name, category))
+        markup = '<b>%s</b><small> (%s)</small>' % tuple(args)
+
+        self.__label_preview_result.set_markup(markup)
 
     # pylint: disable-msg=C0103
     def _on_radiobutton_supplied_remote_toggled(self, radio_button=None):
@@ -283,15 +273,11 @@ class RemoteControlProperties(object):
             self.selected_remote = self.supplied_remote
 
     def _on_button_download_clicked(self, button=None):
-        '''
-        Handle clicks on the auto-detection button.
-        '''
+        '''Handle clicks on the auto-detection button.'''
 
         @thread_callback
         def on_download_progress(progress, total, action):
-            '''
-            Handle progress reports from download service.
-            '''
+            '''Handle progress reports from download service.'''
 
             args = (
                 locale.format(percent='%.1f', grouping=True, value=progress/1024.0),
@@ -304,32 +290,33 @@ class RemoteControlProperties(object):
             progress_window.update(progress, total, message)
 
         @thread_callback
-        def on_download_finished(filename, response):
-            '''
-            Handle finished downloads.
-            '''
+        def on_download_success(content, headers):
+            '''Handle finished downloads.'''
 
             self.__dialog.set_sensitive(True)
             progress_window.hide()
 
-            if 'application/x-gzip' == response['content-type']:
+            if 'application/x-gzip' == headers.get('content-type'):
                 try:
-                    backend.get_service().InstallRemoteDatabase(filename)
+                    backend.get_service().InstallRemoteDatabase(content.name)
                     self.__update_remotes_db()
 
                 except dbus.DBusException, ex:
                     show_message(self.__dialog, progress_window.title, ex.message)
 
+            elif httplib.NOT_MODIFIED == getattr(content, 'code', None):
+                show_message(self.__dialog, progress_window.title,
+                             details=_('No updates available. Your remote control configuration ' +
+                                       'files are already up-to-date.'),
+                             message_type=gtk.MESSAGE_INFO)
+
             else:
-                show_message(self.__dialog,
-                             progress_window.title,
-                             _('Download of updated receiver configurations failed.'))
+                show_message(self.__dialog, progress_window.title,
+                             _('Download of updated remote control configurations failed.'))
 
         @thread_callback
         def on_download_failure(message):
-            '''
-            Handle failure reports from download service.
-            '''
+            '''Handle failure reports from download service.'''
 
             self.__dialog.set_sensitive(True)
             progress_window.hide()
@@ -338,15 +325,21 @@ class RemoteControlProperties(object):
 
         self.__dialog.set_sensitive(False)
         progress_window = ProgressWindow(self.__ui)
-        progress_window.show(self.__dialog, _('Updating Custom Remote Configurations'))
+        progress_window.show(self.__dialog, _('Updating Remote Configuration Files'))
 
         gtk.gdk.threads_leave()
 
         try:
-            net.retrieve_tarball(config.URI_UPDATES,
+            timestamp = (
+                os.path.isfile(config.LIRC_REMOTES_TARBALL) and
+                os.path.getmtime(config.LIRC_REMOTES_TARBALL) or
+                None)
+
+            net.retrieve_tarball(tarball_uri=config.URI_UPDATES,
                                  progress_callback=on_download_progress,
-                                 finished_callback=on_download_finished,
-                                 failure_callback=on_download_failure)
+                                 success_callback=on_download_success,
+                                 failure_callback=on_download_failure,
+                                 reference_time=timestamp)
 
         finally:
             gtk.gdk.threads_enter()
@@ -375,9 +368,7 @@ class RemoteControlProperties(object):
         self.__combo_receiver_product_list.set_active(0)
 
     def __setup_devices_model(self, device_nodes):
-        '''
-        Populate the combo box for device with device nodes.
-        '''
+        '''Populate the combo box for device with device nodes.'''
 
         self.__combo_device.show()
         self.__spinbutton_device.hide()
@@ -411,9 +402,7 @@ class RemoteControlProperties(object):
 
 
     def __setup_numeric_device(self, device_node):
-        '''
-        Initialize the spin-button for numeric device selection.
-        '''
+        '''Initialize the spin-button for numeric device selection.'''
 
         device_node = self.__hardware_manager.parse_numeric_device_node(device_node)
         label, value, minimum, maximum = device_node
@@ -431,25 +420,29 @@ class RemoteControlProperties(object):
 
         self._on_spinbutton_device_value_changed()
 
+    def _on_combo_device_changed(self, combo=None):
+        '''Handle changes to device combo-box.'''
+
+        receiver = self.selected_receiver
+        device = self.selected_device
+
+        self.selected_receiver = receiver, device
+
     # pylint: disable-msg=C0103,W0613
     def _on_spinbutton_device_value_changed(self, spinbutton=None):
-        '''
-        Handle changes to the spin-button for numeric device selection.
-        '''
+        '''Handle changes to the spin-button for numeric device selection.'''
 
         self.selected_device = ('%d' % self.__spinbutton_device.get_value())
 
     def _on_product_list_changed(self, product_list):
-        '''
-        Handle selection changes to receiver product list.
-        '''
+        '''Handle selection changes to receiver product list.'''
 
         # lookup selection:
         tree_iter = product_list.get_active_iter()
         receiver, = product_list.get_model().get(tree_iter, 1)
 
-        if receiver:
-            self._begin_update_configuration()
+        #  freeze configuration updates:
+        self._begin_update_configuration()
 
         try:
             # resolve device nodes:
@@ -472,17 +465,11 @@ class RemoteControlProperties(object):
                 self.selected_remote = self.supplied_remote
 
         finally:
-            # TODO: Also update configuration file on device-entry changes
-            # TODO: Should we do anything with the configuration
-            # if the remote or device is changed to ""/None?
-
-            if receiver:
-                self._end_update_configuration(receiver, self.selected_device)
+            #  thaw configuration updates:
+            self._end_update_configuration(receiver, self.selected_device)
 
     def _on_remote_vendor_list_changed(self, vendor_list):
-        '''
-        Handle selection changes to remote vendor list.
-        '''
+        '''Handle selection changes to remote vendor list.'''
 
         tree_iter = vendor_list.get_active_iter()
 
@@ -492,9 +479,7 @@ class RemoteControlProperties(object):
             self.__combo_remote_product_list.set_active(0)
 
     def _on_remote_product_list_changed(self, product_list):
-        '''
-        Handle selection changes to remote product list.
-        '''
+        '''Handle selection changes to remote product list.'''
 
         tree_iter = product_list.get_active_iter()
         remote, = product_list.get_model().get(tree_iter, 1)
@@ -504,9 +489,7 @@ class RemoteControlProperties(object):
             self._end_update_configuration(remote)
 
     def _on_search_progress(self, manager, fraction, message):
-        '''
-        Handle progress reports from the auto-detection process.
-        '''
+        '''Handle progress reports from the auto-detection process.'''
 
         self.__table_receiver_selection.set_sensitive(False)
         self.__hbox_auto_detect_progress.show()
@@ -520,9 +503,7 @@ class RemoteControlProperties(object):
         self.__progressbar_auto_detect.set_text(' %s ' % message)
 
     def _on_search_finished(self, manager = None):
-        '''
-        Handle end of the auto-detection process.
-        '''
+        '''Handle end of the auto-detection process.'''
 
         self.__table_receiver_selection.set_sensitive(True)
         self.__hbox_auto_detect_progress.hide()
@@ -546,25 +527,21 @@ class RemoteControlProperties(object):
                 show_message(self.__dialog,
                              _('No IR Receivers Found'),
                              _('Could not find any IR receiver. Is your device attached?\n\n' +
-                             'Also notice that some devices, such as homebrew serial port ' +
-                             'receivers, must be selected manually since there is no ' +
+                             'Note that some devices, such as homebrew serial port ' +
+                             'receivers must be selected manually since there is no ' +
                              'way to detect them automatically.'),
                              buttons=responses)):
 
                 self._on_button_auto_detect_clicked()
 
     def _on_receiver_found(self, manager, receiver, udi, device):
-        '''
-        Update the list of auto-detected receivers, when a new one was found.
-        '''
+        '''Update the list of auto-detected receivers, when a new one was found.'''
 
         if self.__receiver_chooser.append(receiver, udi, device) > 1:
             self.__receiver_chooser.show()
 
     def _select_chosen_receiver(self):
-        '''
-        Choose the receiver selected in the auto-detection list.
-        '''
+        '''Choose the receiver selected in the auto-detection list.'''
         self.selected_receiver = self.__receiver_chooser.selected_receiver
 
         if self.supplied_remote:
@@ -601,6 +578,9 @@ class RemoteControlProperties(object):
                         device.update_configuration(service, *args)
                         device = None
 
+                    elif len(args):
+                        service.ManageLircDaemon('disable')
+
                     if 0 == self.__configuration_level:
                         # This can throw an AccessDeniedException exeption:
                         service.ManageLircDaemon('restart')
@@ -640,10 +620,13 @@ class RemoteControlProperties(object):
         except dbus.DBusException, ex:
             logging.error(ex)
 
+    def __write_receiver_configuration(self, service, receiver, device):
+        pass
+    def __write_remote_configuration(self, service, remote):
+        pass
+
     def _on_receiver_chooser_dialog_response(self, dialog, response):
-        '''
-        Handle confirmed selections in the chooser for auto-detected receivers.
-        '''
+        '''Handle confirmed selections in the chooser for auto-detected receivers.'''
 
         if gtk.RESPONSE_ACCEPT == response:
             self._select_chosen_receiver()
@@ -651,12 +634,10 @@ class RemoteControlProperties(object):
         dialog.hide()
 
     def _on_button_auto_detect_clicked(self, button=None):
-        '''
-        Handle clicks on the auto-detection button.
-        '''
+        '''Handle clicks on the auto-detection button.'''
 
         # bring user interface to initial state:
-        self._on_search_progress(None, 0, _('Searching remote controls...'))
+        self._on_search_progress(None, 0, _('Searching for remote controls...'))
 
         if not self.__receiver_chooser:
             self.__receiver_chooser = ReceiverChooserDialog(self.__ui)
@@ -667,16 +648,12 @@ class RemoteControlProperties(object):
         self.__hardware_manager.search_receivers()
 
     def _on_auto_detect_stop_button_clicked(self, button):
-        '''
-        Handle clicks on the auto-detection's "Cancel" button.
-        '''
+        '''Handle clicks on the auto-detection's "Cancel" button.'''
 
         self.__hardware_manager.cancel()
 
     def _on_custom_configuration_button_clicked(self, button):
-        '''
-        Handle clicks on the auto-detection's "Custom Configuration" button.
-        '''
+        '''Handle clicks on the auto-detection's "Custom Configuration" button.'''
 
         if not self.__custom_configuration:
             self.__custom_configuration = CustomConfiguration(self.__ui)
@@ -686,16 +663,12 @@ class RemoteControlProperties(object):
                                         self.selected_remote)
 
     def _on_button_close_clicked(self, button):
-        '''
-        Handle clicks on the "Close" button.
-        '''
+        '''Handle clicks on the "Close" button.'''
 
         self.__dialog.hide()
 
     def _on_button_unlock_clicked(self, button):
-        '''
-        Handle clicks on the "Unlock" button.
-        '''
+        '''Handle clicks on the "Unlock" button.'''
 
         granted = self._unlock()
 
@@ -745,9 +718,7 @@ class RemoteControlProperties(object):
         return granted
 
     def _set_widgets_locked(self, locked):
-        '''
-        Gray (or ungray) widgets which require PolicyKit authorization.
-        '''
+        '''Gray (or ungray) widgets which require PolicyKit authorization.'''
         self.__ui.get_widget('vbox').set_sensitive(not locked)
 
         # Gray out the Unlock button if we are now already unlocked:
@@ -756,9 +727,7 @@ class RemoteControlProperties(object):
 
     # pylint: disable-msg=R0201
     def _on_button_help_clicked(self, button):
-        '''
-        Handle "Help" button clicks.
-        '''
+        '''Handle "Help" button clicks.'''
 
         args = 'yelp', 'ghelp:gnome-lirc-properties'
         env = {'DISPLAY': self.__dialog.get_screen().make_display_name()}
@@ -773,24 +742,18 @@ class RemoteControlProperties(object):
 
     # pylint: disable-msg=R0201
     def _on_window_hide(self, window):
-        '''
-        React when the dialog is hidden.
-        '''
+        '''React when the dialog is hidden.'''
 
         # Stop the main loop so that run() returns:
         gtk.main_quit()
 
     def _on_lirc_properties_dialog_realize(self, dialog):
-        '''
-        Start services which need the dialog from being realized.
-        '''
+        '''Start services which need the dialog from being realized.'''
 
         self.__key_listener.start()
 
     def run(self):
-        '''
-        Show the dialog and return when the window is hidden.
-        '''
+        '''Show the dialog and return when the window is hidden.'''
 
         self.__dialog.connect('hide', self._on_window_hide)
         self.__dialog.show()
@@ -798,9 +761,7 @@ class RemoteControlProperties(object):
         gtk.main()
 
     def __get_selected_receiver(self):
-        '''
-        Retrieve the currently selected receiver.
-        '''
+        '''Retrieve the currently selected receiver.'''
 
         tree_iter = self.__combo_receiver_product_list.get_active_iter()
         receiver = None
@@ -811,6 +772,14 @@ class RemoteControlProperties(object):
         return receiver
 
     def __set_selected_receiver(self, receiver):
+        '''
+        Select the specified receiver.
+
+        The receiver can be specified as Receiver object,
+        or as tuple consiting of vendor and product name.
+        Additionaly the device node can be passed.
+        '''
+
         self._begin_update_configuration()
 
         vendor_name, product_name, device = None, None, None
@@ -855,9 +824,7 @@ class RemoteControlProperties(object):
             self._end_update_configuration(receiver, device)
 
     def __get_selected_remote(self):
-        '''
-        Retrieve the currently selected remote.
-        '''
+        '''Retrieve the currently selected remote.'''
 
         tree_iter = self.__combo_remote_product_list.get_active_iter()
         remote = None
@@ -914,35 +881,26 @@ class RemoteControlProperties(object):
             self.__combo_remote_product_list.set_active_iter(tree_iter)
 
     def __get_selected_device(self):
-        '''
-        Retrieve the currently selected device.
-        '''
-
+        '''Retrieve the currently selected device.'''
         return self.__entry_device.get_text().strip()
 
     def __set_selected_device(self, device_node):
-        '''
-        Change the currently selected device.
-        '''
-
+        '''Change the currently selected device.'''
         self.__entry_device.set_text(device_node)
 
     def __get_supplied_remote(self):
-        '''
-        Retrieve the supplied remot of the currently selected remote.
-        '''
+        '''Retrieve the supplied remot of the currently selected remote.'''
 
         receiver = self.selected_receiver
 
         if receiver:
+            # pylint: disable-msg=E1103
             return receiver.find_supplied_remote(self.__remotes_db)
 
         return None
 
     def __set_use_supplied_remote(self, use_supplied_remote):
-        '''
-        Chooses if the supplied remote shall be used.
-        '''
+        '''Chooses if the supplied remote shall be used.'''
 
         if use_supplied_remote:
             self.__radiobutton_supplied_remote.set_active(True)
@@ -951,9 +909,7 @@ class RemoteControlProperties(object):
             self.__radiobutton_other_remote.set_active(True)
 
     def __get_use_supplied_remote(self):
-        '''
-        Checks if the supplied remote shall be used.
-        '''
+        '''Checks if the supplied remote shall be used.'''
 
         return self.__radiobutton_supplied_remote.get_active()
 
