@@ -20,9 +20,11 @@ User interface related services.
 '''
 
 # publish common functions
-from gnome_lirc_properties.ui.common                  import show_message
+# pylint: disable-msg=W0401
+from gnome_lirc_properties.ui.common                  import *
 
 # publish dialog classes
 from gnome_lirc_properties.ui.CustomConfiguration     import CustomConfiguration
+from gnome_lirc_properties.ui.ProgressWindow          import ProgressWindow
 from gnome_lirc_properties.ui.ReceiverChooserDialog   import ReceiverChooserDialog
 from gnome_lirc_properties.ui.RemoteControlProperties import RemoteControlProperties
